/*
 * QQQ - Low-code Application Framework for Engineers.
 * Copyright (C) 2021-2022.  Kingsrook, LLC
 * 651 N Broad St Ste 205 # 6917 | Middletown DE 19709 | United States
 * contact@kingsrook.com
 * https://github.com/Kingsrook/
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package com.kingsrook.qqq.backend.core.model.session;


import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;


/*******************************************************************************
 **
 *******************************************************************************/
public class QSession implements Serializable
{
   private String idReference;
   private QUser  user;
   private String uuid;

   // implementation-specific custom values
   private Map<String, String> values;



   /*******************************************************************************
    ** Default constructor, puts a uuid in the session
    **
    *******************************************************************************/
   public QSession()
   {
      this.uuid = UUID.randomUUID().toString();
   }



   /*******************************************************************************
    ** Getter for idReference
    **
    *******************************************************************************/
   public String getIdReference()
   {
      return idReference;
   }



   /*******************************************************************************
    ** Setter for idReference
    **
    *******************************************************************************/
   public void setIdReference(String idReference)
   {
      this.idReference = idReference;
   }



   /*******************************************************************************
    ** Getter for user
    **
    *******************************************************************************/
   public QUser getUser()
   {
      return user;
   }



   /*******************************************************************************
    ** Setter for user
    **
    *******************************************************************************/
   public void setUser(QUser user)
   {
      this.user = user;
   }



   /*******************************************************************************
    **
    *******************************************************************************/
   public String getValue(String key)
   {
      if(values == null)
      {
         return null;
      }
      return values.get(key);
   }



   /*******************************************************************************
    **
    *******************************************************************************/
   public void setValue(String key, String value)
   {
      if(values == null)
      {
         values = new HashMap<>();
      }
      values.put(key, value);
   }



   /*******************************************************************************
    **
    *******************************************************************************/
   public QSession withValue(String key, String value)
   {
      if(values == null)
      {
         values = new HashMap<>();
      }
      values.put(key, value);
      return (this);
   }



   /*******************************************************************************
    **
    *******************************************************************************/
   public Map<String, String> getValues()
   {
      return values;
   }



   /*******************************************************************************
    **
    *******************************************************************************/
   public void setValues(Map<String, String> values)
   {
      this.values = values;
   }



   /*******************************************************************************
    ** Getter for uuid
    **
    *******************************************************************************/
   public String getUuid()
   {
      return uuid;
   }



   /*******************************************************************************
    ** Setter for uuid
    **
    *******************************************************************************/
   public void setUuid(String uuid)
   {
      this.uuid = uuid;
   }
}
